'use strict';

var getCssDataDocumentation = require('../utils/get-css-data-documentation.cjs');
var css = require('@master/css');
var cssDataProvider = require('../utils/css-data-provider.cjs');
var getUtilityInfo = require('../utils/get-utility-info.cjs');

function inspectSyntax(document, position) {
    const classPosition = this.getClassPosition(document, position);
    if (!classPosition) return;
    const { token } = classPosition;
    const range = {
        start: document.positionAt(classPosition.range.start),
        end: document.positionAt(classPosition.range.end)
    };
    const styleClasses = this.css.styles[token];
    if (styleClasses) {
        const documentation = getCssDataDocumentation.getCSSDataDocumentation({}, {
            generatedCSS: css.generateCSS([
                token
            ], this.css),
            docs: '/guide/styles'
        });
        if (documentation) {
            return {
                contents: {
                    kind: documentation.kind,
                    value: `(style) ` + styleClasses.join(' ') + '\n' + documentation.value
                }
            };
        }
    } else {
        const rules = this.css.generate(token);
        const rule = rules[0];
        if (rule) {
            if (rule.layer === css.Layer.Utility) {
                const { data, docs } = getUtilityInfo(rule.RegisteredRule, this.css);
                const documentation = getCssDataDocumentation.getCSSDataDocumentation(data, {
                    generatedCSS: css.generateCSS([
                        token
                    ], this.css),
                    docs
                });
                if (documentation) {
                    return {
                        contents: documentation,
                        range
                    };
                }
            } else {
                const nativeProperties = cssDataProvider.provideProperties();
                const nativeCSSPropertyData = nativeProperties.find(({ name })=>name === rule.id);
                const documentation = getCssDataDocumentation.getCSSDataDocumentation(nativeCSSPropertyData, {
                    generatedCSS: css.generateCSS([
                        token
                    ], this.css),
                    docs: '/reference/' + (css.isCoreRule(rule.id) && rule.id)
                });
                if (documentation) {
                    return {
                        contents: documentation,
                        range
                    };
                }
            }
        }
    }
}

module.exports = inspectSyntax;
