'use strict';

var css = require('@master/css');
var cssDataProvider = require('./css-data-provider.cjs');
var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var sortCompletionItems = require('./sort-completion-items.cjs');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');

const kind = vscodeLanguageserverProtocol.CompletionItemKind.Function;
function getPseudoElementCompletionItems(css$1 = new css.MasterCSS(), syntax) {
    const pseudoElementDataList = cssDataProvider.providePseudoElements();
    const completionItems = pseudoElementDataList.map((data)=>{
        // fix https://github.com/microsoft/vscode-custom-data/issues/78
        const name = /::(?:part|slotted)/.test(data.name) ? data.name + '()' : data.name;
        let sortText = name.startsWith('::-') ? 'zzzz' + name.slice(3) : 'zz' + name.replace(/^::/, '');
        if (sortText.endsWith('()')) sortText = 'z' + sortText;
        return {
            label: name,
            sortText,
            documentation: getCssDataDocumentation.getCSSDataDocumentation(data, {
                generatedCSS: css.generateCSS([
                    syntax + name.slice(2)
                ], css$1),
                docs: '/reference/syntaxes#selectors'
            }),
            kind,
            data
        };
    });
    for(const selectorName in css$1.config.selectors){
        if (!selectorName.startsWith('::')) continue;
        const selectorValue = css$1.config.selectors[selectorName];
        const name = selectorName.endsWith('(') ? selectorName + ')' : selectorName;
        const value = typeof selectorValue === 'string' ? selectorValue.endsWith('(') ? selectorValue + ')' : selectorValue : selectorValue;
        const data = pseudoElementDataList.find((data)=>typeof value === 'string' ? value.startsWith(data.name) : false);
        let sortText = name.startsWith('::-') ? 'zzzz' + name.slice(3) : 'zz' + name.replace(/^::/, '');
        if (sortText.endsWith('()')) sortText = 'z' + sortText;
        const completionItem = {
            label: name,
            documentation: getCssDataDocumentation.getCSSDataDocumentation(data, {
                generatedCSS: css.generateCSS([
                    syntax + name.slice(2)
                ], css$1),
                docs: '/reference/syntaxes#selectors'
            }),
            sortText,
            kind,
            detail: String(value),
            data
        };
        completionItems.push(completionItem);
    }
    return sortCompletionItems(completionItems);
}

module.exports = getPseudoElementCompletionItems;
