import { generateCSS, MasterCSS } from '@master/css';
import cssDataProvider from './css-data-provider.mjs';
import { CompletionItemKind } from 'vscode-languageserver-protocol';
import sortCompletionItems from './sort-completion-items.mjs';
import { getCSSDataDocumentation } from './get-css-data-documentation.mjs';

const kind = CompletionItemKind.Function;
function getPseudoElementCompletionItems(css = new MasterCSS(), syntax) {
    const pseudoElementDataList = cssDataProvider.providePseudoElements();
    const completionItems = pseudoElementDataList.map((data)=>{
        // fix https://github.com/microsoft/vscode-custom-data/issues/78
        const name = /::(?:part|slotted)/.test(data.name) ? data.name + '()' : data.name;
        let sortText = name.startsWith('::-') ? 'zzzz' + name.slice(3) : 'zz' + name.replace(/^::/, '');
        if (sortText.endsWith('()')) sortText = 'z' + sortText;
        return {
            label: name,
            sortText,
            documentation: getCSSDataDocumentation(data, {
                generatedCSS: generateCSS([
                    syntax + name.slice(2)
                ], css),
                docs: '/reference/syntaxes#selectors'
            }),
            kind,
            data
        };
    });
    for(const selectorName in css.config.selectors){
        if (!selectorName.startsWith('::')) continue;
        const selectorValue = css.config.selectors[selectorName];
        const name = selectorName.endsWith('(') ? selectorName + ')' : selectorName;
        const value = typeof selectorValue === 'string' ? selectorValue.endsWith('(') ? selectorValue + ')' : selectorValue : selectorValue;
        const data = pseudoElementDataList.find((data)=>typeof value === 'string' ? value.startsWith(data.name) : false);
        let sortText = name.startsWith('::-') ? 'zzzz' + name.slice(3) : 'zz' + name.replace(/^::/, '');
        if (sortText.endsWith('()')) sortText = 'z' + sortText;
        const completionItem = {
            label: name,
            documentation: getCSSDataDocumentation(data, {
                generatedCSS: generateCSS([
                    syntax + name.slice(2)
                ], css),
                docs: '/reference/syntaxes#selectors'
            }),
            sortText,
            kind,
            detail: String(value),
            data
        };
        completionItems.push(completionItem);
    }
    return sortCompletionItems(completionItems);
}

export { getPseudoElementCompletionItems as default };
